/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Gnu General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Gnu General Public License for more details.
 *
 * You should have received a copy of the Gnu General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.antsim.impl;

import anthill.*;
import anthill.factory.*;
import anthill.antsim.*;
import anthill.util.*;

/**
 *  Class <CODE>ExperimentImpl</CODE> is a simple implementation of 
 *  <CODE>Experiment</CODE> which repeat the execution of a single
 *  scenario a certain number of times and returns a single
 *  Stats object containing a summary of the statistics for all the 
 *  executions.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class ExperimentImpl
implements Experiment
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  The identifier of the counter used in a <CODE>Stats</CODE> object to 
   *  count the number of repetitions of the scenario evaluation in a 
   *  experiment.
   */
  public static final String NUMBER_OF_ROUNDS_ID = "NumberOfRounds";

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Factory used to instantiate objects */
  protected Factory factory;
  
  /** Class name of the scenario to be used */
  protected String scenarioClass;
  
  /** Number of times a scenario is evaluated  */
  protected int iterations;
  
  /** Scenario on which evaluate the experiment */
  protected Scenario scenario;
  
  /** Version identifier of the scenario; when init params change, the version increase */
  protected int scenarioVersion;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a new experiment by setting the given number of
   *  times by which a scenario has to be evaluated.
   *  
   *  @param factory the factory to be used to create scenarios
   *  @param iterations the number of scenario evaluations to be performed
   */
  public ExperimentImpl(Factory factory, int iterations)
  {
    this(factory, Scenario.class.getName(), iterations);
  }

  /**
   *  Instantiates a new experiment by setting the given number of
   *  times by which a scenario of the specified class has to be evaluated.
   *
   *  @param factory the factory to be used to create scenarios
   *  @param scenarioClass the class name of the scenario object to be
   *    created
   *  @param iterations the number of scenario evaluations to be performed
   */
  protected ExperimentImpl(Factory factory, String scenarioClass, int iterations)
  {
    this.factory = factory;
    this.iterations = iterations;
    this.scenarioClass = scenarioClass;
    clear();
  }
  
   
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Experiment
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Perform the evaluation of the current ant algorithm. Returns a
   *  <CODE>Stats</CODE> array containing the statistics collected during the
   *  simulation. 
   *  <p>
   *  In this implementation, a single <CODE>Stats</CODE> object is created
   *  and is used to collect statistics for all scenario executions
   *  performed by this <CODE>Experiment</CODE>.
   */
  public Stats[] evaluate()
  {
    Stats stats = (Stats) factory.get(Stats.class);
    stats.add(NUMBER_OF_ROUNDS_ID, iterations);
    for (int i=0; i < iterations; i++) {
      scenario.clear(); 
      scenario.setStats(stats);
      scenario.evaluate();
    }
    return new Stats[] { stats };
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Restores the object to its initial state. 
   */
  public void clear()
  {
    int version = factory.getVersion(scenarioClass);
    if (scenario == null || scenarioVersion != version) {
      /* Creates the scenario */
      scenario = (Scenario) factory.get(scenarioClass);
    } else {
      /* Restore state of existing scenario */
      scenario.clear();
    }
  }

} // END ExperimentImpl

