/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Gnu General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Gnu General Public License for more details.
 *
 * You should have received a copy of the Gnu General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.antsim.impl;

import anthill.*;
import anthill.nest.*;
import anthill.factory.*;
import anthill.antsim.*;

import java.util.Random;

/**
 *  This is a basic implementation of the Peernet interface. A <CODE>PeernetImpl</CODE>
 *  is characterized by a certain size (i.e., the number of nests composing
 *  it) and a degree (i.e., the number of links connecting it to other
 *  nests). 
 *  <p>
 *  This <CODE>Peernet</CODE> implementation uses the <CODE>Factory</CODE> class to 
 *  dynamically instantiate objects implementing Nest; the <CODE>Factory</CODE> 
 *  instance must be opportunely instructed with an XML definition for <CODE>Nest</CODE>.
 *  
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class PeernetImpl
implements Peernet
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Random number generator */
  private static Random random = new Random();
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Reference to the peernet factory which created this object */
  protected Factory factory;
  
  /** The size of the network, i.e. the number of nests included in it */
  protected int size;

  /** The degree of the nests, i.e. the number of links to other nests */
  protected int degree;

  /** Nest network */
  protected Nest[] nests;

  /** Nest version */
  protected int nestVersion;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Initialize a new Peernet factory
   *  
   *  @param factory the factory to be utilized
   */
  public PeernetImpl(Factory factory, int size, int degree)
  {
    this.factory = factory;
    this.size = size;
    this.degree = degree;
    clear();
  }
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Peernet
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Return the nodes composing the simulated p2p network
   */
  public Nest[] getNests()
  {
    return nests;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Restores the object to its initial state. 
   * New nests are instantiated when this method is called for the first
   * time; if the initialization arguments for the Nest interface have
   * been changed, the old nests are discarded and the new nests are 
   * created. In all other cases, the old nodes are cleared.
   */
  public void clear()
  {

    /*
     * Creates or clears the nodes of the peernet
     */
    int version = factory.getVersion(Nest.class);
    if (nests == null || version != nestVersion) {

      /* Update nestVersion */
      nestVersion = version;
      
      /* Create new nodes */
      nests = new Nest[size];
      for (int i=0; i < size; i++) {
        nests[i] = (Nest) factory.get(Nest.class);
      }

    } else {

      /* Clear old nodes */
      for (int i=0; i < size; i++) {
        nests[i].clear();
      }

    }
      
    /*
     *  Create arcs
     */
    for (int i=0; i < size; i++) {
      for (int j=0; j < degree; j++) {
        int k = random.nextInt(size);
        nests[k].addClientNeighbour(nests[i].getId());
        nests[i].addClientNeighbour(nests[k].getId());
      }
    }
  }

} // END PeernetImpl

