/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Gnu General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Gnu General Public License for more details.
 *
 * You should have received a copy of the Gnu General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.antsim.impl;

import anthill.*;
import anthill.factory.*;
import anthill.antsim.*;
import anthill.util.*;
import java.util.*;

/**
 *  <CODE>RandomResourceSet</CODE> is a simple implementation of <CODE>ResourceSet</CODE>
 *  which generates <CODE>RandomResource</CODE>s. Each random resource has a generic
 *  name, which is the string representing a random long value. Requests are obtained
 *  by selecting randomly one of the resources stored in this <CODE>ResourceSet</CODE>
 *  and by returning the string containing the resource name.
 *  
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class RandomResourceSet
implements ResourceSet, Clearable
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Random generator */
  private static Random random = new Random();

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Number of resources contained in this resource set */
  protected int size;
  
  /** Resources contained in this resource set */
  protected Resource[] resources;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor and clear() method
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Initializes this <CODE>RandomResourceSet</CODE> by creating 
   *  <CODE>size</CODE> resources.
   */
  public RandomResourceSet(int size)
  {
    this.size = size;
    
    resources = new Resource[size];
    for (int i=0; i < size; i++) {
      resources[i] = new RandomResource();
    }
    
  }
    
  /**
   *  Restores the object to its initial state.
   *  In this implementation, do nothing, as the resource set is
   *  immutable.
   */
  public void clear()
  {
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Return the number of resources generated and stored in
   *  this <CODE>ResourceSet</CODE>.
   */
  public int size()
  {
    return size;
  }

  /**
   *  Return an array containing the resources generated and 
   *  stored in this <CODE>ResourceSet</CODE>.
   */
  public Resource[] getResources()
  {
    return resources;
  }
  
  /**
   *  Generate a random request. Depending on the implementation of 
   *  this class, the request could be the identifier of the one of 
   *  the resources contained in this <CODE>ResourceSet</CODE>,
   *  or a more specific request (for example, a free query based on
   *  keywords, or an insert request).
   */
  public Object getRequest()
  {
    int pos = random.nextInt(size);
    return resources[pos].getName();
  }

} // END RandomResourceSet

