/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Gnu General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Gnu General Public License for more details.
 *
 * You should have received a copy of the Gnu General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.genie.impl;

import anthill.*;
import anthill.factory.*;
import anthill.genie.*;
import anthill.antsim.*;
import anthill.antsim.impl.*;
import anthill.util.*;

/**
 *  Class ExperimentImpl is a simple implementation of Experiment.
 *  It is constituted by a set of scenarios, on which the fitness
 *  is computed as the average fitness as computed in each scenario.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class GeneticExperimentImpl
extends ExperimentImpl
implements GeneticExperiment
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Genetic code to be evaluated */
  private GeneticCode code;
  
  /** Evaluated fitness */
  private double fitness;
  
  /** Boolean flag; true if the fitness has been already computed */
  protected boolean computed;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates a new experiment by creating the specified
   *  number of scenarios through the specified scenario factory.
   *
   */
  public GeneticExperimentImpl(Factory factory, int size)
  {
    super(factory, GeneticScenario.class.getName(), size);
    fitness = 0.0;
    computed = false;
    code = null;
  }
  
   
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Experiment
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Set the genetic code to be evaluated. In this basic implementation,
   *  only one genetic code can be present in the Nest network at a time.
   *  If this method is called more than once, the values is substituted
   *  with the new values.
   */
  public void addGeneticCode(GeneticCode code)
  {
    this.code = code;
    
  }

  /**
   *  Execute the experiment, i.e. evaluate the fitness of the genetic code on
   *  each of the scenarios contained in this experimnt. The fitness of the
   *  genetic code for this experiment is obtained by the sum of the fitness 
   *  values obtained by evaluating them on the scenarios.
   *
   *  @throws IllegalStateException if no genetic code is present
   */
  public Stats[] evaluate()
  {

    if (this.code == null)
      throw new IllegalStateException("No genetic code to be evaluated");

    Stats stats = (Stats) factory.get(Stats.class);
    stats.add(NUMBER_OF_ROUNDS_ID, iterations);
    GeneticScenario gs = (GeneticScenario) scenario;
    for (int i=0; i < iterations; i++) {
      gs.clear(); 
      gs.addGeneticCode(code);
      gs.setStats(stats);
      gs.evaluate();
    }
    
    /* Fitness evalation using an external class */
    Fitness fitnessEval = (Fitness) factory.get(Fitness.class);
    fitness = fitnessEval.compute(stats);
    Log.println(200, "Genetic code: " + code + " found " + fitness + "% resources");
    computed = true;

    return new Stats[] { stats };
  }

  
  /**
   *  Returns the evaluated fitness. 
   *
   *  @throws IllegalStateException if the fitness value has not been computed
   *    yet.
   */
  public double getFitness()
  {
    if (!computed)
      throw new IllegalStateException("No fitness value yet!");
    return fitness;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Restores the object to its initial state. Method clear() is invoked on
   *  all scenarios included in this experiment. The genetic code to be
   *  evaluated and the computed fitness are restored to their initial
   *  state.
   */
  public void clear()
  {
    code = null;
    fitness = 0.0;
    computed = false;
    super.clear();
  }

} // END ExperimentImpl

