/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Gnu General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Gnu General Public License for more details.
 *
 * You should have received a copy of the Gnu General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.nestor;

import java.io.*;
import anthill.*;
import anthill.nest.*;

/**
 *  Class LocalAntView is a simple implementation of interface 
 *  <CODE>AntView</CODE>, which implements methods included in
 *  it by invoking the opportune methods on the local nest.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class LocalAntView
implements AntView
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
 
  /** Reference to the nest object hosting this view */
  private LocalNest nest;
  
  /** The descriptor of the ant currently associated with this view */
  private AntDescriptor desc;

  /** True if the resource method move() has been invoked successfully */
  private boolean moved;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////


  /**
   *  Instantiates an AntView and stores in it the references to
   *  which the operations are delegated.
   *  <p>
   *  The constructor has package visibility to avoid
   *  instances not builded through the appropriate factory.
   *
   *  @param storage  the resource database
   *  @param gate     the communication handler
   *  @param desc     the descriptor of the ant to which this AntView
   *                  is associated
   */
  LocalAntView(LocalNest nest, AntDescriptor desc)
  {
    this.nest = nest;
    this.desc = desc;
    moved = false;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for requesting general information
  ////////////////////////////////////////////////////////////////////////////////////////////

  /*  
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the nest for its identifier
   */
  public NestId getNestId()
  {
    return nest.getId();
  }

  
  /*  
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the associated ant descriptor for the nest identifier
   */
  public NestId getLastVisited()
  {
    return desc.getLastVisited();
  }

  /*  
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the associated ant descriptor for the ant identifier
   */
  public AntId getAntId()
  {
    return desc.getId();
  }

  
  /*
   *  Javadoc comment inherited by interface anthill.AntView
   *  Asks the associated ant descriptor for the ant reference
   */
  public Ant getAnt()
  {
    return desc.getAnt();
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for ant movement and creation
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  FIXME: Implement.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Manager</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void addAnt(Ant ant)
  {
  }

  /**
   *  Moves an ant to the specified nest. In this particular implementation,
   *  an ant can perform at most one move operation when in a nest. This 
   *  restriction is imposed in order to avoid unbounded ant replication.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   *
   *  @param id the id of the destination nest
   *  @throws IOException if the ant cannot move to the
   *    destination nest for communication problems.
   */
  public void move(NestId id)
    throws IOException
  {
    nest.getGate().send(id, desc);
    /*
     *  Note that moved is updated only if we are
     *  sure that the send operation has not
     *  raised an exception.
     */
    moved = true;
  }

  /**
   *  Leave trail information on the <CODE>Trail</CODE> object stored
   *  in the nest. If absent, the trail is created. Return true if the
   *  the information is left
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Trail</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public boolean storePreviousNest()
  {
    Trail trail = nest.getPreviousNest(true);
    return trail.add(desc.getId(), desc.getLastVisited());
  }

  /**
   *  Lookup trail information on the <CODE>Trail</CODE> object stored
   *  in the nest and move to the obtained nest. If the trail is absent, 
   *  <CODE>IllegalStateException</CODE> is raised.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Trail</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public NestId getPreviousNest()
  {
    Trail trail = nest.getPreviousNest(false);
    if (trail != null) {
      NestId nid = trail.lookup(desc.getId());
      if (nid != null) {
        if (nid == nest.getId()) {
          /* We have reached the requestor nest; return null */
          return null;
        } else { 
          /* We are not in the requestor nest; just return the id of the prev nest */
          return nid;
        }
      } else { 
        throw new IllegalStateException("No trail information for ant " + desc.getId());
      }
    } else {
      throw new IllegalStateException("No trail information in this nest");
    }
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for neighbour management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Add the new neighbor information to the <CODE>Gate</CODE> object contained
   *  in the current nest. FIXME: correct implementation by removing addAntNeighbour
   *  and adding the method to Gate.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void addNeighbour(NestId id)
    throws IOException
  {
    nest.addAntNeighbour(id);
  }
    
  /**
   *  FIXME: Implement.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the class documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void removeNeighbour(NestId id)
  {
    // nest.removeAntNeighbour(id);
  }
    
  
  /**
   *  Ants invoke this method to obtain the list of nest neighbors
   *  known to the current nest. FIXME: correct implementation to refer to Gate
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Gate</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public Neighbour[] getNeighbours()
  {
    return nest.getNeighbours();
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for pheromone management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Get the pheromone information object related to the species of the
   *  ant associated to this <CODE>AntView</CODE> object. Different
   *  ant species are associated to different pheromone objects.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public Object getPheromone()
  {
    return nest.getPheromone(desc.getAnt().getClass());
  }

  
  /**
   *  Set the pheromone information object related to the species of the
   *  ant associated to this <CODE>AntView</CODE> object. Different
   *  ant species are associated to different pheromone objects.
   *  
   *  @param pheromone the pheromone object to be set.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void setPheromone(Object pheromone)
  {
    nest.setPheromone(pheromone, desc.getAnt().getClass());
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for resource management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Performs the specified request on the local nest and
   *  (possibly) returns the resources satisfying the 
   *  request. Method <CODE>getResources</CODE> is invoked
   *  on the <CODE>Storage</CODE> object included in the nest.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Storage</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   *
   *  @param request the request to be performed
   */
  public Resource[] getResources(Object request)
  {
    return nest.getStorage().getResources(request);
  }

  
  /**
   *  This method is invoked by ants to notify the nest that
   *  the ant associated to this AntView, created to satisfy
   *  a request, is returning the specified set of resources.
   *
   *  @param request the request to be performed
   *  @param resources  the resources satisfying the query
   */
  public void returnResources(Resource[] resources)
  {
    nest.getRequestSet().returnResources(desc.getId(), resources);
  }

  
  /**
   *  Add the specified resource to the local nest. 
   *  Method <CODE>addAntResource</CODE> is invoked
   *  on the <CODE>Storage</CODE> object included in the nest.
   *  <p>
   *  The exact behavior of this method depends on the class implementing
   *  the <CODE>Storage</CODE> interface currently used in the current nest.
   *  Please refer to the implementation documentation for more details.
   *  <p>
   *  Please see the corresponding method comment in interface 
   *  <CODE>AntView</CODE> for additional information.
   */
  public void addResource(Resource resource)
  {
    nest.getStorage().addAntResource(resource);
  }
  
} // END LocalAntView

