/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Gnu General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Gnu General Public License for more details.
 *
 * You should have received a copy of the Gnu General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.nestor;

import anthill.*;
import anthill.nest.*;
import anthill.factory.*;

import java.util.*;


/**
 *  Class <CODE>LocalTrail</CODE> is the implementation of <CODE>Trail</CODE>
 *  to be used during off-line simulations. Programmers are enabled to specify
 *  the maximum number of associations (ant identifier, previous nest identifier)
 *  which can be stored in a <CODE>LocalTrail</CODE> object, or the maximum duration 
 *  of such an associations in a <CODE>LocalTrail</CODE> object, or both.
 *  Assocations which exceed the maximum capacity or are older than the maximum
 *  duration are discarded from this <CODE>LocalTrail</CODE>.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class LocalTrail
extends Thread  
implements Trail
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Max size of the trail information */
  private int maxsize;

  /** Max duration of the trail information */
  private int maxtime;
  
  /** Polling interval */
  private int interval;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Size of the trail information */
  private int size;
  
  /** 
   *  Hash table mapping ant identifiers to the nest from which the ant 
   *  has arrived.
   */
  protected HashMap map;
  
  /** Head of the list connecting the pairs; pairs will be removed here */
  protected Pair head;
  
  /** Tail of the list connecting the pairs; pairs will be added here */
  protected Pair tail;
  
  /** True if the pooling thread has to be stopped */
  private boolean ended = false;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Instantiate a new Trail object by setting the maximum number of
   *  pair (ant id, source) which may be contained in it.
   *
   *  @param maxsize maximum number of pairs (ant id, source) which may
   *    be contained in this <CODE>Trail</CODE> object. If zero, the 
   *    number of pairs is not limited.
   *  @param maxtime maximum time (in milliseconds) of permanence
   *    of a pair in this <CODE>Trail</CODE> object. If zero, the
   *    maximum time of permanence is not limited.
   *  @param interval polling time for obsolete pair removals 
   */
  public LocalTrail(int maxsize, int maxtime, int interval)
  {
    this.maxsize = maxsize;
    this.maxtime = maxtime;
    this.interval = interval;
    map = new HashMap();
    if (maxtime > 0) {
      this.start();
    }
  }
    
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Add a pair (id, source) to the trail object containing
   *  the host. Returns true if the ant identifier is not already
   *  included in this Trail object; return false otherwise.
   *  The "false" notification can be used to detect duplicated
   *  visit to a nest.
   */
  public synchronized boolean add(AntId id, NestId source)
  {
    
    if (map.get(id) == null) {
      /* Check if there is a maximum size for the data structure */
      if (maxsize > 0 && size >= maxsize) {
        /* If the list is not empty, remove the oldest element */
        if (head != null)  {
          head = head.next;
          size--;
        }
      }
      
      /* Add a pair containing the ant id, the nest id and 
       * the current time to the list */
      Pair pair = new Pair(id, source, tail);
      if (tail != null)
        tail.next = pair;
      tail = pair;
      size++;
      
      /* Add the corresponding association to the map */
      map.put(id, pair);
      return true;
    } else {
      return false;
    }
  }

  
  /**
   *  Lookup this <CODE>LocalTrail</CODE> object in order to obtain the 
   *  identifier of the nest from which the specified 
   *  ant arrived. The pair associating the ant
   *  identifier to the source nest is removed from
   *  both the hash table and the linked list.
   */
  public synchronized NestId lookup(AntId id)
  {
    Pair pair = (Pair) map.get(id);
    if (pair != null) {
      
      // FIXME The removal should be configurable
      /* Remove the map from the hash table */
      map.remove(id);

      /* Remove the pair from the list */
      if (pair.prev != null) {
        /* Add a link to the next in the previous */
        pair.prev.next = pair.next;
      } else {
        /* No prev, modify the head */
        head = pair.next;
      }
      if (pair.next != null) {
        /* Add a link to the previous in the next */
        pair.next.prev = pair.prev;
      } else {
        /* No next, modify the tail */
        tail = pair.prev;
      }
        
      // Update size
      size--;
    }
    
    /* Return the nest identifier */
    return (pair == null ? null : pair.nest);
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Run and related methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Periodically scans the linked list of pairs in order
   * to remove staled pairs (ant id, nest id).
   */
  public void run()
  {
    while (!ended) {
      clean();
      try {
        Thread.sleep(interval);
      } catch (InterruptedException e) {
        
      }
    }
  }
  
  /** 
   * Scans the linked list of pairs in order
   * to remove staled pairs (ant id, nest id). 
   */
  private synchronized void clean()
  {
    long expiration = System.currentTimeMillis()-maxtime;
    while (head != null && head.time < expiration) {
      map.remove(head.ant);
      head = head.next;
    }
  }
  
  /**
   *  Close the thread associated to this trail.
   */
  public void end()
  {
    ended = true;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Inner class
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   *  The Pair class is an inner class used to link objects
   *  in the hash table in a list.
   */
  protected class Pair
  {
    
    /** Identifier of the ant which visited this nest */
    protected AntId ant;
    
    /** Identifier of the nest from which this nest has arrived */
    protected NestId nest;
    
    /** Next pair in the linked list */
    protected Pair next;
    
    /** Next pair in the linked list */
    protected Pair prev;

    /** Creation time; used to detect when a link is stale */
    protected long time;
    
    /**
     *  Initializing constructor
     */
    protected Pair(AntId ant, NestId nest, Pair prev)
    {
      this.ant = ant;
      this.nest = nest;
      this.prev = prev;
      this.time = System.currentTimeMillis();
    }
    
  } // END Pair
  
} // END LocalTrail

