/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Gnu General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Gnu General Public License for more details.
 *
 * You should have received a copy of the Gnu General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package random;

import anthill.*;
import java.io.*;

/**
 *  This class contains the implementation of a simple ant which performs
 *  a random walk through the network. Ants move forward until they reach
 *  the searched resource; then, they move backward along the incoming
 *  path. When they reach the requestor nest, they copy the resource 
 *  locally.
 *
 *  @author Alberto Montresor
 *  @version $REVISION$
 */
public class RandomAnt
implements Ant, Serializable
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////

  private static java.util.Random rand;
  static {
    rand = new java.util.Random();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The query associated to this ant */
  private String query;
  
  /** The resource found */
  private Resource resource;
  
  /** True if this ant is returning to the requestor nest */
  private boolean backward;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiate a new RandomAnt by setting the query to be satisfied and the
   *  hash value to be searched.
   */
  public RandomAnt(String query)
  {
    this.query = query;
    resource = null;
    backward = false;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Ant
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Returns the identifier of the "species" of this ant.
   *  Different ant species may have access to different
   *  pheromone information in nests.
   */
  public String getSpecies()
  {
    return "RandomAnt";
  }

  /**
   *  This method encapsulate the computation performed by the
   *  ant. 
   *
   *  @param view reference to the AntView object which
   *    constitutes the controlled environment of the ant
   */
  public void run(AntView view)
  {
    if (backward) {
      goBackward(view);
    } else {
      goForward(view);
    }
  }

  /**
   *  Algorithm used when moving forward
   */
  private void goForward(AntView view)
  {
    /* Leave a trail */
    boolean firsttime = view.storePreviousNest();
    if (!firsttime) {
      /* We have already visited this nest */
      try {
        view.move(view.getLastVisited());
      } catch (Exception e) {
        // Do nothing;
      }
      return;
    }
    
    /* Local query of the database */
    Resource[] resources = view.getResources(query);
    if (resources != null) {
      /* Found resource; go backward */
      resource = resources[0]; 
      backward = true;
      goBackward(view);
      return;
    } 

    /* Selection of the next hop */
    Neighbour[] neighbours = view.getNeighbours();
    int next = rand.nextInt(neighbours.length);
    try {
      view.move(neighbours[next].getId());
    } catch (Exception e) {
      // Do nothing
    }
  }

  
  /**
   *  Algorithm used when moving backward
   */
  private void goBackward(AntView view)
  {
    /** Obtain the previous nest*/
    NestId prev = view.getPreviousNest();
    
    if (prev == null) {
      /** We have reached the requestor nest */
      view.returnResources(new Resource[] { resource });
      view.addResource(resource);
    } else {
      /** The ants move to the previous hop*/
      try {
        view.move(prev);
      } catch (Exception e) {
        // Do nothing
      }
    }
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Externalization methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void writeExternal(ObjectOutput out)
    throws IOException
  {
    // FIXME To be implemented
  }
  
  public void readExternal(ObjectInput in)
  {
    // FIXME To be implemented
  }
  
  
} // END RandomAnt


